/*
 * Copyright (c) 2001-2002 Conexant Systems, Inc.
 * 
 * 1.   Permitted use. Redistribution and use in source and binary forms,
 * with or without modification, are permitted under the terms set forth
 * herein.
 * 
 * 2.   Disclaimer of Warranties. CONEXANT AND OTHER CONTRIBUTORS MAKE NO
 * REPRESENTATION ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.
 * IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTIES OF ANY KIND.
 * CONEXANT AND OTHER CONTRIBUTORS DISCLAIMS ALL WARRANTIES WITH REGARD TO
 * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE, GOOD TITLE AND AGAINST INFRINGEMENT.
 * 
 * This software has not been formally tested, and there is no guarantee that
 * it is free of errors including, but not limited to, bugs, defects,
 * interrupted operation, or unexpected results. Any use of this software is
 * at user's own risk.
 * 
 * 3.   No Liability.
 * 
 * (a) Conexant or contributors shall not be responsible for any loss or
 * damage to Company, its customers, or any third parties for any reason
 * whatsoever, and CONEXANT OR CONTRIBUTORS SHALL NOT BE LIABLE FOR ANY
 * ACTUAL, DIRECT, INDIRECT, SPECIAL, PUNITIVE, INCIDENTAL, OR CONSEQUENTIAL
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED, WHETHER IN CONTRACT, STRICT OR OTHER LEGAL THEORY OF
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
 * WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 * 
 * (b) User agrees to hold Conexant and contributors harmless from any
 * liability, loss, cost, damage or expense, including attorney's fees,
 * as a result of any claims which may be made by any person, including
 * but not limited to User, its agents and employees, its customers, or
 * any third parties that arise out of or result from the manufacture,
 * delivery, actual or alleged ownership, performance, use, operation
 * or possession of the software furnished hereunder, whether such claims
 * are based on negligence, breach of contract, absolute liability or any
 * other legal theory.
 * 
 * 4.   Notices. User hereby agrees not to remove, alter or destroy any
 * copyright, trademark, credits, other proprietary notices or confidential
 * legends placed upon, contained within or associated with the Software,
 * and shall include all such unaltered copyright, trademark, credits,
 * other proprietary notices or confidential legends on or in every copy of
 * the Software.
 * 
 */

/****************************************************************************************
 *                     Version Control Information										*
 *                                                                                      *
 * $Header:   P:/d942/octopus/archives/osinclude/osservices.h_v   1.16   10 Oct 2002 16:22:56   beliloi  $
 * 
*****************************************************************************************/


/****************************************************************************************

File Name:			osservices.h	

File Description:	Prototypes for OS-specific functions.

*****************************************************************************************/


#ifndef __OSSERVICES_H__
#define __OSSERVICES_H__

#include "typedefs.h"
#include "comtypes.h"

/****************************************************************************************


	Atomic operations 


****************************************************************************************/

/****************************************************************************************
    The OsAtomicCompareAndSwap function compares the value at the specified address with 
	oldVal. The value of newValue is written to the address only if oldValue and the 
	value at the address are equal. OSCompareAndSwap returns true if newValue is written 
	to the address; otherwise, it returns false. 
    
    Params:
        oldValue The value to compare at address.
        newValue The value to write to address if oldValue compares true.
        address The 4-byte aligned address of the data to update atomically.
    Result: true if newValue was written to the address. 
****************************************************************************************/
BOOL OsAtomicCompareAndSwap (PVOID oldValue, PVOID newValue, PVOID* address);


/****************************************************************************************
    OsAtomicAdd
   
    OsAtomicAdd function adds the specified amount to the value at the specified 
    address and returns the result.
    Params:
        amount  The amount to add.
        address The 4-byte aligned address of the value to update atomically.
    Result: The result of the addition.
****************************************************************************************/
INT32	OsAtomicAdd   (INT32 amount, INT32* address);


/****************************************************************************************
    OsAtomicIncrement function increments the value at the specified address by one and 
	returns the new value.
    Param: address The 4-byte aligned address of the value to update atomically.
    Result: The result of the increment.
****************************************************************************************/
INT32	OsAtomicIncrement (INT32* address);


/****************************************************************************************
    OsAtomicDecrement function decrements the value at the specified address by one and 
	returns the new value.
    Param: address The 4-byte aligned address of the value to update atomically.
    Result: The result of the increment. 
****************************************************************************************/
INT32	OsAtomicDecrement (INT32* address);


/****************************************************************************************


	Synchronisation objects


**********************************************************************************/

typedef struct tagHLOCK
{
	int dummy;

} *HLOCK;


/****************************************************************************************
    OsLockCreate 
    Allocates and initializes Lock object (CriticalSection, Semaphore, Mutex - 
    whatever you call it) that are general purpose blocking mutual exclusion locks

Parameters :	None
Return value :	Handle to the created Lock object to be passed as parameter for OS 
				functions acting on locks. Returns NULL on failure.
Interrupt safe: no
****************************************************************************************/
HLOCK   OsLockCreate(void);


/****************************************************************************************
    OsLockDestroy
    Destroys Lock Object. Any blocked waiters will not be woken.
    Parameters :	hLock	-	Handle identifing the Lock object.

Return value :	None
Functionality : Destroys a critical instance. 
Interrupt safe: no
****************************************************************************************/
void    OsLockDestroy(HLOCK hLock);


/****************************************************************************************
    OsLockLock
    If the lock is held by any thread, block waiting for its unlock. 
    This function may block and so should not be called from interrupt level. 
    Locking same object recursively from one thread may result in deadlock. 
    
Parameters :	hLock	-	Handle identifing the Lock object.
Return value :	None
Interrupt safe: no
****************************************************************************************/
void	OsLockLock(HLOCK hLock);


/****************************************************************************************
    OsLockTry
    Lock if it is currently unlocked, and return true. 
    If the lock is held by any thread, return false.
    
Parameters :	hLock	-	Handle identifing the Lock object.
Return value :	None
Interrupt safe: no
****************************************************************************************/
BOOL OsLockTry(HLOCK hLock);


/****************************************************************************************

    OsLockUnlock
    Unlock the Lock and wake any blocked waiters. 
    Results are undefined if the caller has not locked the Lock. 
    
Parameters :	hLock	-	Handle identifing the Lock object.
Return value :	None
Interrupt safe: no
****************************************************************************************/
void	OsLockUnlock(HLOCK hLock);


typedef struct tagHINTERRUPT
{
	int dummy;

} *HINTERRUPT;


/****************************************************************************************
    OsInterruptsDisable
    Disable interrupts and preemption to synchronize between interrupt context and thread 
	context. Cannot be held across any blocking operation, and should be held for very 
	short periods.
    
Parameters :	None
Return value :	Handle to previous interrupt state
Interrupt safe: yes
****************************************************************************************/
HINTERRUPT  OsInterruptsDisable(void);


/****************************************************************************************
    OsInterruptsRestore
    Restores preemption and interrupts to the state as they were when 
	OsInterruptsDisable() was called

Parameters :	Interrupt state handle
Interrupt safe: yes
****************************************************************************************/
void        OsInterruptsRestore(HINTERRUPT hPreviousState);


/****************************************************************************************


	Memory/string operations


****************************************************************************************/

void    OsMemSet    (PVOID pBuf, UINT8 c, UINT32 Count);
PVOID	OsMemCpy    (PVOID pDest, PVOID pSrc, UINT32 Count);
int		OsMemCmp    (PVOID pBuff1, PVOID pBuff2, UINT32 Count);

PVOID	OsStrCpy    (LPSTR szDest, LPCSTR szSrc);
PVOID	OsStrnCpy   (LPSTR szDest, LPCSTR szSrc, int MaxSize);
int     OsStrCmp    (LPCSTR szStr1, LPCSTR szStr2);
int	    OsStrLen    (LPCSTR szStr);
int	    OsToupper   (int c);
int	    OsTolower   (int c);
int	    OsIsDigit   (int c);
#ifndef TARGET_HCF_STARFISH_VXWORKS
int     OsSprintf   (LPSTR buffer, LPCSTR format, ...);
#else
#define OsSprintf sprintf
#endif
typedef struct
{
	int dummy;
} *HOSTIMER;




/**********************************************************************************


	Misc utilities


**********************************************************************************/


/**********************************************************************************
Function name :	OsGetSystemTime
Parameters :	None
Return value :	returns timestamp in milliseconds
Functionality	The function returnes a milliseconds timestamp.
Interrupt safe: yes
**********************************************************************************/
UINT32	OsGetSystemTime(void);


/**********************************************************************************
Function name :	OsSleep
Parameters :	Time in milliseconds to put the thread to sleep
Return value :	None
Functionality	Put the executing thread to sleep
Interrupt safe: no
**********************************************************************************/
void	OsSleep (UINT32 sleepTime);


/**********************************************************************************
Function name :	OsGetCurrentThread
Parameters :	None
Return value :	Thread ID of the active thread
Functionality:	Returns thread ID of the thread running the code
Interrupt safe: no
**********************************************************************************/
typedef struct tagHTHREAD
{
	int dummy;

} *HTHREAD;

HTHREAD OsGetCurrentThread(void);

/**********************************************************************************
Function name :	OSCallOnMyStack
Parameters :	pFunc			-	function to be called
				lParam			-	parameter to pass for pFunc
				pTopOfStack		-	Pointer to stack
				StackSizeBytes	-	stack size in bytes.
Return value :	None
Functionality :	Temporarily switches a stack to a different locked block of memory.
				This function is intended to be used in an environment with limited
				stack size given to an interrupt service routine. 
Interrupt safe: yes
**********************************************************************************/
void	OsCallOnMyStack	(	IN	PCBFUNC	pFunc, 
							IN	UINT32	lParam, 
							IN	PUINT32	pTopOfStack, 
							IN	UINT32	StackSizeBytes);




/****************************************************************************************


	Timer


****************************************************************************************/


/**********************************************************************************
Function name :	OSCreatePeriodicTimeOut
Parameters :	InitialTimeOut		- initial interval in milliseconds 
									  between 2 timeout callbacks
				pTimeOutCallBack	- Pointer to a fucntion that will be called at 
									  the specified interval
				pRefData			- Reference Data to be passed to callback 
									  function
Return value :	Returns a handle to the periodic timeout. This handle is to be
				passed to OS functions acting on periodic timeout. Returns NULL
				on failure.
Functionality :	Creates a periodic timeout of InitialTimeOut interval. 
				pTimeOutCallBack function will be called on each timer interval. 
Interrupt safe: no
**********************************************************************************/
HOSTIMER	OsCreatePeriodicTimer	(	IN	UINT32			InitialTimeOut,
										IN	PCBFUNC			pTimeOutCallBack,
										IN	PALLOC_FUNC		pFuncAlloc,
										IN	PFREE_FUNC		pFuncFree,
										IN	PVOID			pRefData,
										OUT	HTHREAD*		pThreadId);


/**********************************************************************************
Function name :	OSDesrtoyPeriodicTimeOut
Parameters :	hTimeOut -	Handle to the periodic time out returned by 
							OSCreatePeriodicTimeOut.
Return value :	none
Functionality : destroys a periodic timeout resource.
Interrupt safe: no
**********************************************************************************/
void	OsDestroyPeriodicTimer	(IN	HOSTIMER	hTimeOut);


/**********************************************************************************
Function name :	OSSetPeriodicTimeOut
Parameters :	hTimeOut	-	Handle to the periodic timeout
				NewTimeOut	-	New timeout interval
Return value :	returns TRUE when the timeout interval was successfully changed to 
				NewTimeOut
Functionality : resets the timeout inteval to a new value.
Interrupt safe: no
**********************************************************************************/
BOOL	OsSetPeriodicTimer	(	IN	HOSTIMER	hTimeOut,
								IN	UINT32		NewTimeOut);


/**********************************************************************************
Function name :	OSImmediateTimeOut
Parameters :	hTimeOut	-	Handle to the periodic timeout
Return value :	none
Functionality : Schedules call of the callback function as soon as non-interrupt
				context becomes available.
Interrupt safe: yes
**********************************************************************************/
void    OsImmediateTimeOut	(IN	HOSTIMER	hTimeOut);



/**********************************************************************************


	Debug utilities


**********************************************************************************/


void    OsDebugPrintf       ( LPCSTR szFormat, ...);
void    OsDebugBreakpoint   ( LPCSTR szMsg );

#define OsSwapUINT16(x) ((((x) & 0xFF00) >> 8) | (((x) & 0x00FF) << 8))
#define OsSwapUINT32(x) ((OsSwapUINT16(x) << 16) | (OsSwapUINT16((x) >> 16)))

//Event handling functions
typedef struct OsEventStruct
{
	int dummy;
} *HOSEVENT;

typedef enum WAIT_RESULT_ENUM
{
	WAIT_OK,
	WAIT_TIMEOUT,
	WAIT_ERROR
} WAIT_RESULT;


/********************************************************************************

Name:           OsCreateEvent

Description:    Creates an OS-specific syncronization object with event capability
				
Arguments:      none

Return Value:   handle to created event

Notes:			

********************************************************************************/
HOSEVENT	OsCreateEvent	(void);

/********************************************************************************

Name:           OsDeleteEvent

Description:    Destroys event object
				
Arguments:      hEvent	- handle to event

Return Value:   none

Notes:			

********************************************************************************/
void		OsDeleteEvent	(HOSEVENT hEvent);

/********************************************************************************

Name:           OsWaitForEvent

Description:    waits until event is set by OsSetEvent() call
				
Arguments:      hEvent	- handle to event

Return Value:   none

Notes:			

********************************************************************************/
void		OsWaitForEvent	(HOSEVENT hEvent);

/********************************************************************************

Name:           OsSetEvent

Description:    sets event to signalled state
				
Arguments:      hEvent	- handle to event

Return Value:   none

Notes:			

********************************************************************************/
void		OsSetEvent		(HOSEVENT hEvent);

/********************************************************************************

Name:           OsWaitForEventTime

Description:    waits until event is set by OsSetEvent() call or specified time has passed
				
Arguments:      hEvent	- handle to event
				timeout	- time in ms for function to block the calling thread if event remains
				in non-signalled state

Return Value:   Result of the call
				WAIT_OK			- event gets set
				WAIT_TIMEOUT	- time elapses
				WAIT_ERROR		- error

Notes:			

********************************************************************************/
WAIT_RESULT	OsWaitForEventTime	(HOSEVENT hEvent, UINT32 timeout);

/********************************************************************************

Name:           OsClearEvent

Description:    sets event to non-signalled state
				
Arguments:      hEvent	- handle to event

Return Value:   none

Notes:			

********************************************************************************/
void		OsClearEvent	(HOSEVENT hEvent);

#endif      // #ifndef __OSSERVICES_H__
